//+------------------------------------------------------------------+
//| 9:20 AM 9/10/2017                                     spread.mq4 |
//|                                Copyright 2017-19, William Roeder |
//|                                        mailto:WHRoeder@yahoo.com |
//+------------------------------------------------------------------+
#property   strict
#property   copyright   "Copyright  2017-2019, WHRoeder@yahoo.com"
#property   link        "mailto:WHRoeder@yahoo.com"
#property   description "Spread above candle"
#property   version     "1.1"
/*
double   get_spread_max(){ 
   // Make sure indicator "spread" is attached.
   double NU   = iCustom(_Symbol,_Period,"spread",0,0);

      static const string  nameMax  = _Symbol + "SpreadMax";
   double   value;   GlobalVariableGet(nameMax, value);
   return   value;
}
   double   spreadMax   = get_spread_max();
*/
#include <common_utilities.mqh>  // PRICE/INDEX/COUNT/HeikenAshi
//+------------------------------------------------------------------+
//{ Specific code                                                    |
//+------------------------------------------------------------------+
//---- indicator parameters
input bool  useHeikenAshi  = false;

enum FontSelect{  Arial,
                  Times_New_Roman,  // Times New Roman
                  Courier};
static string font2string[]={"Arial", "Times New Roman", "Courier"};
//--- input parameters
input COUNT             dispWindow     = 0;
input ENUM_BASE_CORNER  corner         = CORNER_LEFT_LOWER;
input FontSelect        selectedFont   = Times_New_Roman;
input COUNT             textSize       = 16;
input bool              bold           = true;
input color             fntcolor       = clrYellow;
input COUNT             XDistance      = 5;
input COUNT             YDistance      = 25;

//---- indicator buffers
#property   indicator_chart_window
#define     SHORTNAME         "Spread"
#property   indicator_buffers 1
//#property indicator_type1   DRAW_LINE      // Bug 1090 must set.
//#property indicator_style1  STYLE_SOLID
//#property indicator_width1  1
//#property indicator_label1  "const"
//{ Shown on chart
//{ Shown on chart, not in data window.
PRICE       bDot[];
#property   indicator_type1   DRAW_ARROW     // Bug 1090 must set.
#property   indicator_label1  "Spread"
#property   indicator_color1  clrDarkGray
#define     INDICATOR_BUFFER1 noData().arr(WD_DOT_SMALL).buf(bDot)
//} Shown on chart, not in data window.
//} Shown on chart
//{ For data window, not shown on chart.
//} For data window, not shown on chart.
//{ extra buffers, not shown anywhere.
// None
//#define   BUFFERS_COUNT_TOTAL  TBD                                 // Extra
//} extra buffers, not shown anywhere.
//+------------------------------------------------------------------+
//} Specific code                                                    |
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
	// #define CREATE_ONINIT
#include <indicator_utilities.mqh>  // OnInitBuffers
string                  sFontType;
int OnInit(){
   if (dispWindow != INV_COUNT){
      sFontType   = font2string[selectedFont] + (bold ? " Bold" : "");
      #define NAME_LABEL "Spread"
      ObjectCreate(NAME_LABEL, OBJ_LABEL, dispWindow, 0, 0);
      ObjectSet(NAME_LABEL, OBJPROP_CORNER, corner);
      ObjectSet(NAME_LABEL, OBJPROP_XDISTANCE, XDistance);
      ObjectSet(NAME_LABEL, OBJPROP_YDISTANCE, YDistance);
      ObjectSetInteger(0, NAME_LABEL, OBJPROP_SELECTABLE, false);
   }
//---
   return OnInitBuffers();
}  // OnInit
//+------------------------------------------------------------------+
//| Custom indicator de-initialization function                      |
//+------------------------------------------------------------------+
void OnDeinit(const int reason){
// EventKillTimer();
   if (dispWindow != INV_COUNT)  ObjectDelete(NAME_LABEL);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){
   //{ Average/Maximum Spread.   (Must run per tick.)
      static const string  nameMax        = _Symbol + "SpreadMax";
      static const string  nameAve        = _Symbol + "SpreadAve";
      static const string  nameCount      = _Symbol + "Spread_C";
      static const string  nameElapsed    = _Symbol + "Spread_E";
      static const double  power          = 20;
      static const COUNT   secondsPerDay  = 86400; // 24*60*60

      static double     spreadMax, spreadAve, nTicks, nSeconds;
      static datetime   then;
      datetime now         = TimeCurrent();
      double   spreadCur   = MathMax(0,Ask - Bid) / _Point;
      if(prev_calculated == 0){
         then  = now;
         if(!GlobalVariableGet(nameMax,     spreadMax)
         || !GlobalVariableGet(nameAve,     spreadAve)
         || !GlobalVariableGet(nameCount,   nTicks)
         || !GlobalVariableGet(nameElapsed, nSeconds)
         || spreadMax == 0.0 || !MathIsValidNumber(spreadMax)
         ){ spreadAve = spreadMax = spreadCur;  nTicks = 0; nSeconds = 1;  }
      }
      nTicks += 1;   nSeconds += SECONDS(now - then); then  = now;
      if(nSeconds > 10*secondsPerDay){ nTicks *= 0.9; nSeconds *= 0.9;  } //EMA
            double   ticksPerDay = secondsPerDay * nTicks / nSeconds;
         double   weight      = MathMin(ticksPerDay, nTicks);
      spreadMax   = PMA(spreadMax, spreadCur, weight, power);
      spreadAve   = EMA(spreadAve, spreadCur, weight);

      GlobalVariableSet(nameMax,  spreadMax);
      GlobalVariableSet(nameAve,  spreadAve);
      GlobalVariableSet(nameCount,   nTicks);
      GlobalVariableSet(nameElapsed, nSeconds);
   //} Average/Maximum Spread.   (Must run per tick.)
   if (dispWindow != INV_COUNT){
      string   display     = StringFormat("Spread %g/%g",
                                 MathRound(spreadAve), MathRound(spreadMax) );
      ObjectSetText(NAME_LABEL, display, textSize, sFontType, fntcolor);
   }

   #define  LAST              0                       // Draw bar zero?
   #define  REDRAW_BAR_LAST   (LAST == 0)             // Redraw bar last.
      static HeikenAshi    ha;
   bDot[LAST]     = spreadMax * _Point + (useHeikenAshi
                                       ? ha.get(HA_TOP, LAST)
                                       : High[LAST] );
   if(bDot[LAST+1] == EMPTY_VALUE)
      bDot[LAST+1]   = spreadMax * _Point + (useHeikenAshi
                                          ? ha.get(HA_TOP, LAST+1)
                                          : High[LAST+1] );
   //----
   return rates_total-LAST-REDRAW_BAR_LAST;
}  // OnCalculate
//+------------------------------------------------------------------+
